using System;
using System.Collections.Generic;
using System.Text;

namespace OSMPreComputation
{
    class blockdetection
    {
        #region Cycle detection for directed graph

        public IRouteCycleClass CycleDetection(IRouteArcClass pArcCls, bool bDirected)
        {
            //search the circumscirbing cycle           
            IRouteCycle pCircumCycle = CircumCycle(pArcCls, bDirected);
            if (pCircumCycle == null)
            {
                MessageBox.Show("No reasonable circumscribing cycle detected in current dataset!\r\n Please check the data and algorithm.", "Warning");
                return null;
            }

            IRouteCycleClass pCycleClass = new RouteCycleClass();
            int[] pIDStatus = new int[pArcCls.ArcCount];
            ArrayList pNewCycles = new ArrayList();

            for (int i = 0; i < pCircumCycle.ArcCount; i++) // set the status of each arc in the circumscribing cycle.
                pIDStatus[pCircumCycle.GetArcID(i)] = 1;

            int[] pIDArray = new int[pArcCls.ArcCount]; //for every arc, the previous arc.
            int[] pObsArray = new int[pArcCls.ArcCount];//for every arc, the status whether is obsolete or delete in terms of current cycyle.
            bool[] pFTArray = new bool[pArcCls.ArcCount]; //for every arc, the current from/to end.

            //pCycleClass.AddCycle(pCircumCycle);
            InwardCycles(pCircumCycle, pNewCycles, ref pIDStatus, pArcCls, bDirected, pIDArray, pObsArray, pFTArray);
            while (pNewCycles.Count > 0)
            {
                IRouteCycle pBaseCycle = pNewCycles[0] as IRouteCycle;
                pCycleClass.AddCycle(pBaseCycle);
                pNewCycles.RemoveAt(0);

                InwardCycles(pBaseCycle, pNewCycles, ref pIDStatus, pArcCls, bDirected, pIDArray, pObsArray, pFTArray);
            }

            return pCycleClass;
        }

        //From the circumscribing cycle, search inward cycles.
        /// <summary>
        /// Search inward cycles.
        /// </summary>
        /// <param name="pBaseCycle"></param>
        /// <param name="bCurrRight"></param>
        /// <param name="pNewCycles"></param>
        /// <param name="pIDStatus"></param>
        private void InwardCycles(IRouteCycle pBaseCycle, ArrayList pNewCycles, ref int[] pIDStatus, IRouteArcClass pArcCls, bool bDirected, int[] pIDArray, int[] pObsArray, bool[] pFTArray)
        {
            bool bCurrRight = !pBaseCycle.Right;
            for (int i = 0; i < pBaseCycle.ArcCount; i++)
            {
                int pCurrArcID = pBaseCycle.GetArcID(i);
                if (pIDStatus[pCurrArcID] == 2) continue;
                pIDStatus[pCurrArcID] = 2; //important

                bool bCurrArcFT = pBaseCycle.GetArcFT(i);
                IRouteCycle pCycle = SearchCycle(pCurrArcID, bCurrArcFT, bCurrRight, pArcCls, bDirected, pIDArray, pObsArray, pFTArray);
                if (pCycle == null) continue; //.... need to check?

                pCycle.Level = pBaseCycle.Level + 1;
                pCycle.Right = bCurrRight;
                pNewCycles.Add(pCycle);

                int pNewID = -1;
                for (int j = 0; j < pCycle.ArcCount; j++)
                {
                    pNewID = pCycle.GetArcID(j);
                    if (pIDStatus[pNewID] == 2) continue;

                    pIDStatus[pNewID] += 1;
                }
            }
        }

        private IRouteCycle SearchCycle(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, IRouteArcClass pArcCls, bool bDirected, int[] pIDArray, int[] pObsArray, bool[] pFTArray)
        {
            int pCurrArcID = pOrigArcID;
            bool bCurrArcFT = bOrigArcFT;
            IRouteArc pCurrArc = pArcCls.GetArc(pCurrArcID);
            //if (pCurrArc.IsPartRound) return; // ignore the roundabout arcs.
            if (pCurrArc.GetPoint(0).Overlap(pCurrArc.GetPoint(pCurrArc.PointCount - 1))) return null;
            for (int i = 0; i < pIDArray.Length; i++)
            {
                pObsArray[i] = 0;
                pFTArray[i] = false;
                pIDArray[i] = -1;
            }

            IRouteCycle pCycle = null;
            int pNextArcID = -1;
            bool bNextArcFT = false;
            while (pNextArcID != pOrigArcID)
            {
                ArrayList pConnIDs, pConnFTs, pConnAngles;
                GetRankAngle(pArcCls.GetArc(pCurrArcID), bCurrArcFT, pArcCls, bDirected, out pConnIDs, out pConnFTs, out pConnAngles);
                if (!bOrigRight)     //from right to left(counterclockwisely), the angle changes from -180 to 180
                {
                    pConnIDs.Reverse();
                    pConnFTs.Reverse();
                    pConnAngles.Reverse();
                }

                if (pConnIDs.IndexOf(pOrigArcID) != -1)
                {
                    int bIndex = pConnIDs.IndexOf(pOrigArcID);
                    bool bFT = Convert.ToBoolean(pConnFTs[bIndex]);
                    if (bFT != bOrigArcFT)
                    {
                        pCycle = TrackCycle(pOrigArcID, bOrigArcFT, bOrigRight, pCurrArcID, pIDArray, pFTArray, pArcCls);
                        return pCycle;
                    }

                    pObsArray[pCurrArcID] = -1;
                    pCurrArcID = pOrigArcID;
                    bCurrArcFT = bOrigArcFT;
                    continue;
                }

                //if current arc end connection is 0, obsolete it and back to previous arc
                if (pConnIDs.Count == 0)
                {
                    pObsArray[pCurrArcID] = -1;
                    pCurrArcID = pIDArray[pCurrArcID];
                    if (pCurrArcID == pOrigArcID)
                        bCurrArcFT = bOrigArcFT;
                    else
                        bCurrArcFT = pFTArray[pCurrArcID];
                    continue;
                }

                bool bObsolete = true;
                for (int i = 0; i < pConnIDs.Count; i++) //Always get the first one
                {
                    pNextArcID = Convert.ToInt32(pConnIDs[i]);
                    if (pObsArray[pNextArcID] == -1) continue; //already be searched and obsolete
                    IRouteArc pNextArc = pArcCls.GetArc(pNextArcID);
                    if (pNextArc.GetPoint(0).Overlap(pNextArc.GetPoint(pNextArc.PointCount - 1))) continue;

                    if (pIDArray[pNextArcID] != -1) // self intersect with path, obsolete and back from the intersection/redundant arc
                    {
                        pObsArray[pNextArcID] = -1;
                        pCurrArcID = pIDArray[pNextArcID];
                        if (pCurrArcID == pOrigArcID)
                            bCurrArcFT = bOrigArcFT;
                        else
                            bCurrArcFT = pFTArray[pCurrArcID]; //important
                        bObsolete = false;
                        break;
                    }

                    bNextArcFT = Convert.ToBoolean(pConnFTs[i]);
                    pIDArray[pNextArcID] = pCurrArcID;
                    pFTArray[pNextArcID] = !bNextArcFT;

                    pCurrArcID = pNextArcID;
                    bCurrArcFT = !bNextArcFT;

                    bObsolete = false;
                    break;
                }
                if (bObsolete) //obsolete and back one step from current step.
                {
                    pObsArray[pCurrArcID] = -1;
                    if (pIDArray[pCurrArcID] == -1) MessageBox.Show(pCurrArcID.ToString());
                    pCurrArcID = pIDArray[pCurrArcID]; //important
                    if (pCurrArcID == pOrigArcID)
                        bCurrArcFT = bOrigArcFT;
                    else
                        bCurrArcFT = pFTArray[pCurrArcID]; //important
                }
            }
            return pCycle;
        }

        private IRouteCycle TrackCycle(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, int pLastArcID, int[] pIDArray, bool[] pFTArray, IRouteArcClass pArcCls)
        {
            IRouteCycle pCycle = new RouteCycle();
            ArrayList pIDs = new ArrayList(), pFTs = new ArrayList();

            int pCurrArcID = pLastArcID;
            while (pCurrArcID != pOrigArcID)
            {
                bool bFT = pFTArray[pCurrArcID];
                int pPreID = pIDArray[pCurrArcID];

                pIDs.Add(pCurrArcID);
                pFTs.Add(bFT);

                pCurrArcID = pPreID;
            }

            pIDs.Reverse(); pFTs.Reverse();
            pCycle.AddArc(pOrigArcID, bOrigArcFT);
            for (int i = 0; i < pIDs.Count; i++)
            {
                int pCycleArcID = Convert.ToInt32(pIDs[i]);
                IRouteArc pCycleArc = pArcCls.GetArc(pCycleArcID);
                pCycle.AddArc(pCycleArcID, Convert.ToBoolean(pFTs[i]), pCycleArc as IRouteBound, pCycleArc.Length);
            }
            pCycle.Right = bOrigRight;
            return pCycle;
        }


        //Get circumscribing cycle
        /// <summary>
        /// To get this cycle, we need to head from the outmost arc and turn recursively in both direction, left and right. If we get two different cycles, 
        /// pick up the larger one; if we get two same cycles, pick up one of them. Then we need use the cycle to calcuate how many arcs there are inside the cycle.
        /// If the percent is too litte, e.g. less than 50 percent or 70 percent, we have the reason to suspect that we are not on the main group of the graph which
        /// contains more than one group.
        /// </summary>
        /// <param name="pArcCls"></param>
        /// <param name="bDirected"></param>
        private IRouteCycle CircumCycle(IRouteArcClass pArcCls, bool bDirected)
        {
            int pUpID, pDownID, pLeftID, pRightID;
            ArcsAtBound(pArcCls, out pUpID, out pDownID, out pLeftID, out pRightID);

            IRouteCycle pCircumCycle = null;
            int[] pOutArc = new int[] { pUpID, pDownID, pLeftID, pRightID };
            for (int i = 0; i < pOutArc.Length; i++)
            {
                IRouteArc pArc = pArcCls.GetArc(pOutArc[i]);
                if ((pArc.FromConnCount == 0 && pArc.ToConnCount == 0) || (pArc.Round || pArc.Link)) continue;

                int pGrpNum = SearchGroup(pOutArc[i], pArcCls);
                if (pGrpNum / (double)pArcCls.ArcCount < 0.75) continue;

                bool bOrigArcFT = false, bOrigRight = true;
                if (pArc.ToConnCount == 0) bOrigArcFT = true;

                IRouteCycle pCycle1 = null;
                pCycle1 = SearchCircumCycle(pArc.ID, bOrigArcFT, bOrigRight, pArcCls, bDirected);
                if (pCycle1 != null)
                    pCycle1.Prunes(pArcCls);


                IRouteCycle pCycle2 = null;
                pCycle2 = SearchCircumCycle(pArc.ID, bOrigArcFT, !bOrigRight, pArcCls, bDirected);

                if (pCycle2 != null)
                    pCycle2.Prunes(pArcCls);

                if (pCycle1 == null)
                {
                    if (pCycle2 == null)
                    {
                        MessageBox.Show("Maybe there is no circumscribing cyle or not on the main group.", "Information");
                        //In this case, continue to next outmost arc
                    }
                    else
                        MessageBox.Show("Circumscribing cycle detection wrong: One is null, the other is Not null.", "Warning");
                }
                else
                {
                    if (pCycle2 == null)
                        MessageBox.Show("Circumscribing cycle detection wrong: One is null, the other is Not null.", "Warning");
                    else
                    {
                        if (pCycle1.Equals(pCycle2))
                        {
                            pCircumCycle = pCycle1;
                        }
                        else
                        {
                            if (pCycle1.Contains(pCycle2))
                                pCircumCycle = pCycle1;
                            else if (pCycle2.Contains(pCycle1))
                                pCircumCycle = pCycle2;
                            else
                                MessageBox.Show("Circumscribing cycle detection wrong: Neither equals and nor contains", "Warning");
                        }
                    }
                }                
                if (pCircumCycle != null) break;               
            }


            return pCircumCycle;
        }

        private IRouteCycle SearchCircumCycle(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, IRouteArcClass pArcCls, bool bDirected)
        {
            IRouteCycle pCircumCycle = null;

            int[] pIDArray = new int[pArcCls.ArcCount]; //for every arc, the previous arc.
            bool[] pFTArray = new bool[pArcCls.ArcCount]; //for every arc, the current from/to end.
            int[] pObsArray = new int[pArcCls.ArcCount];//for every arc, the status whether is obsolete or delete in terms of current cycyle.
            for (int i = 0; i < pIDArray.Length; i++) pIDArray[i] = -1;

            ArrayList pDoubleArcIDs = new ArrayList();
            ArrayList pDoubleIDs = new ArrayList();
            ArrayList pDoubleFTs = new ArrayList();

            int pNextArcID = -1, pCurrArcID = pOrigArcID;
            bool bNextArcFT = false, bCurrArcFT = false;
            while (pNextArcID != pOrigArcID)
            {
                ArrayList pConnIDs, pConnFTs, pConnAngles;
                GetRankAngle(pArcCls.GetArc(pCurrArcID), bCurrArcFT, pArcCls, bDirected, out pConnIDs, out pConnFTs, out pConnAngles);
                if (!bOrigRight)     //from right to left(counterclockwisely), the angle changes from -180 to 180
                {
                    pConnIDs.Reverse();
                    pConnFTs.Reverse();
                    pConnAngles.Reverse();
                }

                //if current arc end connection is 0, obsolete it and back to previous arc
                if (pConnIDs.Count == 0)
                {
                    if (pCurrArcID == pOrigArcID)
                        bCurrArcFT = !bCurrArcFT;
                    else
                    {
                        pObsArray[pCurrArcID] = -1;
                        pCurrArcID = pIDArray[pCurrArcID];
                        bCurrArcFT = pFTArray[pCurrArcID];
                    }
                    continue;
                }

                //I am here, the largest cycle! :=)
                if (pConnIDs.IndexOf(pOrigArcID) != -1)
                {
                    pCircumCycle = TrackCircumCycle(pOrigArcID, bOrigArcFT, bOrigRight, pCurrArcID, pIDArray, pFTArray, pDoubleArcIDs, pDoubleIDs, pDoubleFTs, pArcCls);
                    return pCircumCycle;
                }

                bool bObsolete = true;
                for (int i = 0; i < pConnIDs.Count; i++) //Always get the first one
                {
                    pNextArcID = Convert.ToInt32(pConnIDs[i]);
                    if (pObsArray[pNextArcID] == -1) continue; //already be searched and obsolete

                    if (pIDArray[pNextArcID] != -1) // self intersect with path, obsolete and back from the intersection/redundant arc
                    {
                        if (pDoubleArcIDs.IndexOf(pNextArcID) != -1)
                            MessageBox.Show("What's happening in cycle detectiong", "Warning");
                        pDoubleArcIDs.Add(pNextArcID);
                        pDoubleIDs.Add(pIDArray[pNextArcID]);
                        pDoubleFTs.Add(pFTArray[pNextArcID]);
                    }

                    bNextArcFT = Convert.ToBoolean(pConnFTs[i]);
                    pIDArray[pNextArcID] = pCurrArcID;
                    pFTArray[pNextArcID] = !bNextArcFT;

                    pCurrArcID = pNextArcID;
                    bCurrArcFT = !bNextArcFT;

                    bObsolete = false;
                    break;
                }
                if (bObsolete) //obsolete and back one step from current step.
                {
                    pObsArray[pCurrArcID] = -1;
                    pCurrArcID = pIDArray[pCurrArcID];
                    bCurrArcFT = pFTArray[pCurrArcID];
                }
            }
            return pCircumCycle;
        }

        private IRouteCycle TrackCircumCycle(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, int pLastArcID, int[] pIDArray, bool[] pFTArray,
            ArrayList pDoubleArcIDs, ArrayList pDoubleIDs, ArrayList pDoubleFTs, IRouteArcClass pArcCls)
        {
            IRouteCycle pCircumCycle = new RouteCycle();
            ArrayList pIDs = new ArrayList(), pFTs = new ArrayList();

            int pCurrArcID = pLastArcID;
            while (pCurrArcID != pOrigArcID)
            {
                bool bFT = pFTArray[pCurrArcID];
                int pPreID = pIDArray[pCurrArcID];

                int pDoubleIdx = pDoubleArcIDs.IndexOf(pCurrArcID);
                if (pDoubleIdx != -1 && pIDs.IndexOf(pCurrArcID) != -1)
                {
                    bFT = Convert.ToBoolean(pDoubleFTs[pDoubleIdx]);
                    pPreID = Convert.ToInt32(pDoubleIDs[pDoubleIdx]);
                }

                pIDs.Add(pCurrArcID);
                pFTs.Add(bFT);

                pCurrArcID = pPreID;
            }

            pIDs.Reverse(); pFTs.Reverse();
            pCircumCycle.AddArc(pOrigArcID, bOrigArcFT);
            for (int i = 0; i < pIDs.Count; i++)
            {
                int pCycleArcID = Convert.ToInt32(pIDs[i]);
                IRouteArc pCycleArc = pArcCls.GetArc(pCycleArcID);
                pCircumCycle.AddArc(pCycleArcID, Convert.ToBoolean(pFTs[i]), pCycleArc as IRouteBound, pCycleArc.Length);
            }
            pCircumCycle.Right = bOrigRight;
            return pCircumCycle;

            //StreamWriter sr = new StreamWriter("c:\\xxxx.txt");
            //sr.AutoFlush = true;
            //for (int i = 0; i < pIDs.Count; i++)
            //    sr.Write( pIDs[i].ToString() + ",");
            //sr.Close();
        }

        //calculation OSM Arc angles
        private void GetRankAngle(IRouteArc pCurrArc, bool bCurrArcFT, IRouteArcClass pArcCls, bool bDirected, out ArrayList pConnIDs, out ArrayList pConnFTs, out ArrayList pConnAngles)
        {
            pConnIDs = new ArrayList();
            pConnFTs = new ArrayList();
            pConnAngles = new ArrayList();

            int pConnNum = 0;
            if (bCurrArcFT)
                pConnNum = pCurrArc.FromConnCount;
            else
                pConnNum = pCurrArc.ToConnCount;

            if (pConnNum == 0 || (bDirected && pCurrArc.Oneway && bCurrArcFT)) return; //жϷ

            int pConnArcID = 0;
            bool bConnArcFT = false;
            double pConnAngle = 0.0;
            IRouteArc pConnArc = null;
            for (int i = 0; i < pConnNum; i++)
            {
                pConnArcID = pCurrArc.GetConnID(i, bCurrArcFT);
                bConnArcFT = pCurrArc.GetConnFT(i, bCurrArcFT);

                pConnArc = pArcCls.GetArc(pConnArcID);
                if (bDirected && pConnArc.Oneway && !bConnArcFT) continue; //жӵarcǷΪ
                //if (pConnArc.IsPartRound) continue; //roundabout

                pConnIDs.Add(pConnArcID);//ID
                pConnFTs.Add(bConnArcFT);//ӵλ
                pConnAngle = pCurrArc.GetConnAngle(i, bCurrArcFT);// CGFun.CalcArcAngle(pCurrArc, bCurrArcFT, pConnArc, bConnArcFT);
                pConnAngles.Add(pConnAngle);//ӵĽǶ                
            }

            //
            int ID1, ID2;
            bool bFT1, bFT2;
            double pAngle1, pAngle2;
            for (int i = 0; i < pConnAngles.Count; i++)
            {
                for (int j = i + 1; j < pConnAngles.Count; j++)
                {
                    pAngle1 = Convert.ToDouble(pConnAngles[i]);
                    pAngle2 = Convert.ToDouble(pConnAngles[j]);
                    if (pAngle1 > pAngle2)
                    {
                        pConnAngles[i] = pAngle2;
                        pConnAngles[j] = pAngle1;

                        ID1 = Convert.ToInt32(pConnIDs[i]);
                        ID2 = Convert.ToInt32(pConnIDs[j]);
                        pConnIDs[i] = ID2;
                        pConnIDs[j] = ID1;

                        bFT1 = Convert.ToBoolean(pConnFTs[i]);
                        bFT2 = Convert.ToBoolean(pConnFTs[j]);
                        pConnFTs[i] = bFT2;
                        pConnFTs[j] = bFT1;
                    }
                }
            }
        }


        //Get the outmost arc that touches the boundary according to the envelope of OSM dataset
        /// <summary>
        /// Calculate the outmost arc according to the envelope of OSM dataset.
        /// </summary>
        /// <param name="pArcCls">OSM Arc Class</param>
        /// <param name="pOrient">0: up; 1: down; 2: left; 3: right</param>
        /// <returns></returns>
        private void ArcsAtBound(IRouteArcClass pArcCls, out int pUpID, out int pDownID, out int pLeftID, out int pRightID)
        {
            IRouteBound pCurrBound = pArcCls.GetArc(0) as IRouteBound;
            IRouteBound pBound = new RouteBound(pCurrBound.XMin, pCurrBound.YMin, pCurrBound.XMax, pCurrBound.YMax);

            //IRouteArc pArcUp, pArcDown, pArcLeft, pArcRight;
            pUpID = -1; pDownID = -1; pLeftID = -1; pRightID = -1;

            IRouteArc pArc = null;
            int pArcNum = pArcCls.ArcCount;
            for (int i = 1; i < pArcNum; i++)
            {
                pArc = pArcCls.GetArc(i);

                pCurrBound = pArc as IRouteBound;
                if (pBound.XMin > pCurrBound.XMin)
                {
                    pBound.XMin = pCurrBound.XMin;
                    pLeftID = i;
                }
                if (pBound.XMax < pCurrBound.XMax)
                {
                    pBound.XMax = pCurrBound.XMax;
                    pRightID = i;
                }
                if (pBound.YMin > pCurrBound.YMin)
                {
                    pBound.YMin = pCurrBound.YMin;
                    pDownID = i;
                }
                if (pBound.YMax < pCurrBound.YMax)
                {
                    pBound.YMax = pCurrBound.YMax;
                    pUpID = i;
                }
            }
        }


        //Search the group members number to which current arc belongs, also could be changed to get the group member IDs.
        ///<summary>
        /// Search the group members number to which current arc belongs
        /// </summary>
        /// <returns></returns>
        public static int SearchGroup(int pCurrArcID, IRouteArcClass pArcCls)
        {
            int[] pVisitedArray = new int[pArcCls.ArcCount];
            int pVisitedNumber = 1;
            pVisitedArray[pCurrArcID] = 1;

            ArrayList pConnIDs = new ArrayList(), pConnFTs = new ArrayList();
            IRouteArc pCurrArc = pArcCls.GetArc(pCurrArcID);
            SpanConnect(pCurrArc, true, ref pVisitedArray, ref pVisitedNumber, pConnIDs, pConnFTs);
            SpanConnect(pCurrArc, false, ref pVisitedArray, ref pVisitedNumber, pConnIDs, pConnFTs);

            while (pConnIDs.Count > 0)
            {
                int[] pConnIDArray = (int[])pConnIDs.ToArray(typeof(int));
                bool[] pConnFTArray = (bool[])pConnFTs.ToArray(typeof(bool));
                pConnIDs.Clear();
                pConnFTs.Clear();
                for (int i = 0; i < pConnIDArray.Length; i++)
                {
                    pCurrArc = pArcCls.GetArc(pConnIDArray[i]);
                    SpanConnect(pCurrArc, !pConnFTArray[i], ref pVisitedArray, ref pVisitedNumber, pConnIDs, pConnFTs);
                }
            }
            return pVisitedNumber;
        }

        private static void SpanConnect(IRouteArc pCurrArc, bool bCurrArcFT, ref int[] pVisitedArray, ref int pVisitedNumber, ArrayList pConnIDs, ArrayList pConnFTs)
        {
            int pArcConnNum = 0;
            if (bCurrArcFT)
                pArcConnNum = pCurrArc.FromConnCount;
            else
                pArcConnNum = pCurrArc.ToConnCount;

            for (int i = 0; i < pArcConnNum; i++)
            {
                int pConnArcID = pCurrArc.GetConnID(i, bCurrArcFT);
                if (pVisitedArray[pConnArcID] == 0 && pConnIDs.IndexOf(pConnArcID) == -1)
                {
                    pVisitedArray[pConnArcID] = 1;
                    pVisitedNumber++;

                    pConnIDs.Add(pConnArcID);
                    pConnFTs.Add(pCurrArc.GetConnFT(i, bCurrArcFT));
                }
            }
        }


        // Update the arcs in graph
        /// <summary>
        /// Set the cycle ID to every arc
        /// </summary>
        /// <param name="pArcCls"></param>
        /// <param name="pCycleClass"></param>
        public void UpdateArcGraph(ref IRouteArcClass pArcCls, IRouteCycleClass pCycleClass)
        {
            //Update the graph arc class files: left and right cycle ID.
            IRouteCycle pCycle = null;
            for (int i = 0; i < pCycleClass.CycleCount; i++)
            {
                pCycle = pCycleClass.GetCycle(i);

                for (int j = 0; j < pCycle.ArcCount; j++)
                {
                    IRouteArc pArc = pArcCls.GetArc(pCycle.GetArcID(j));
                    if (pArc.RightCycleID != -1 && pArc.LeftCycleID != -1) MessageBox.Show("Something unexpected in updating cycle info.", "Warning");

                    if (pArc.RightCycleID == -1)
                        pArc.RightCycleID = pCycle.ID;
                    else if (pArc.LeftCycleID == -1)
                        pArc.LeftCycleID = pCycle.ID;
                } // end of cycle
            }
        }


        #region Recursive methods to get the circumscribing and inner cycles, which are not suitable in terms of huge network and obsolete accordingly.

        private void InwardCycles(IRouteCycle pBaseCycle, bool bRight, ArrayList pNewCycles, ArrayList pViewedIDs, ArrayList pClosedIDs, IRouteArcClass pArcCls, bool bDirected)
        {
            int pCurrArcID = 0;
            bool bCurrArcFT = false;
            for (int i = 0; i < pBaseCycle.ArcCount; i++)
            {
                pCurrArcID = pBaseCycle.GetArcID(i);
                if (pClosedIDs.IndexOf(pCurrArcID) != -1) continue;

                pClosedIDs.Add(pCurrArcID);
                bCurrArcFT = pBaseCycle.GetArcFT(i);

                IRouteCycle pCycle = null;
                string pIDStr = pCurrArcID + ",", pFTStr = bCurrArcFT + ",";
                RecurCycle(pCurrArcID, bCurrArcFT, bRight, pCurrArcID, bCurrArcFT, pIDStr, pFTStr, bDirected, pArcCls, ref pCycle, ref pClosedIDs);

                if (pCycle == null) continue;

                pCycle.Level = pBaseCycle.Level + 1;
                pNewCycles.Add(pCycle);

                int pNewID = -1;
                for (int j = 0; j < pCycle.ArcCount; j++)
                {
                    pNewID = pCycle.GetArcID(j);

                    if (pClosedIDs.IndexOf(pNewID) != -1) continue; //No need to recure if the part of base cycle has been traversed.

                    if (pBaseCycle.IndexOfArc(pNewID) != -1)
                    {
                        pClosedIDs.Add(pNewID);
                        continue;
                    }

                    if (pViewedIDs.IndexOf(pNewID) != -1)
                    {
                        pViewedIDs.Remove(pNewID);
                        pClosedIDs.Add(pNewID);
                    }
                    else
                    {
                        pViewedIDs.Add(pNewID);
                    }
                }
            }
        }

        /// <summary>
        /// From the base cycle, jut keep turning inversly, then it is impossible to  hit self before closing, that's one of the advantages except efficiency.
        /// </summary>
        /// <param name="pOrigArcID"></param>
        /// <param name="bOrigArcFT"></param>
        /// <param name="bOrigRight"></param>
        /// <param name="pCurrArcID"></param>
        /// <param name="bCurrArcFT"></param>
        /// <param name="pIDStr"></param>
        /// <param name="pFTStr"></param>
        /// <param name="bDirected"></param>
        /// <param name="pArcCls"></param>
        /// <param name="pCycle"></param>
        /// <param name="pClosedIDs"></param>
        private void RecurCycle(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, int pCurrArcID, bool bCurrArcFT, string pIDStr, string pFTStr,
            bool bDirected, IRouteArcClass pArcCls, ref IRouteCycle pCycle, ref ArrayList pClosedIDs)
        {
            IRouteArc pCurrArc = pArcCls.GetArc(pCurrArcID);
            //if (pCurrArc.IsPartRound) return; // Neglect the roundabout arcs.
            if (pCurrArc.GetPoint(0).Overlap(pCurrArc.GetPoint(pCurrArc.PointCount - 1))) return;

            ArrayList pConnIDs, pConnFTs, pConnAngles;
            GetRankAngle(pCurrArc, bCurrArcFT, pArcCls, bDirected, out pConnIDs, out pConnFTs, out pConnAngles);
            if (pConnIDs.Count == 0) return;

            if (!bOrigRight)
            {
                pConnIDs.Reverse();
                pConnFTs.Reverse();
                pConnAngles.Reverse();
            }

            for (int i = 0; i < pConnIDs.Count; i++)
            {
                int pNextArcID = Convert.ToInt32(pConnIDs[i]);
                bool bNextArcFT = Convert.ToBoolean(pConnFTs[i]);

                IRouteArc pNextArc = pArcCls.GetArc(pNextArcID);
                if (pNextArc.IsConnIDIn(pOrigArcID, !bNextArcFT))
                {
                    pIDStr += pNextArcID + ",";
                    pFTStr += !bNextArcFT + ",";
                    Arcs2Cycle(pIDStr, pFTStr, bOrigRight, ref pCycle, pArcCls);
                    return;
                }
                else
                {
                    if (HitPath(pIDStr, pNextArc, !bNextArcFT)) //Hit path: bending inward/outward to the existing pathno longer implying cycle.
                        return;
                }

                //FlashDetection(pIDStr, pFTStr, pNextArcID, pArcCls); //Animation

                pIDStr += pNextArcID + ",";
                pFTStr += !bNextArcFT + ",";
                if (pIDStr.Length > 8000) MessageBox.Show("what.....");

                RecurCycle(pOrigArcID, bOrigArcFT, bOrigRight, pNextArcID, !bNextArcFT, pIDStr, pFTStr, bDirected, pArcCls, ref pCycle, ref pClosedIDs);
                if (pCycle != null) return;

                if (pIDStr.IndexOf(",") == pIDStr.LastIndexOf(",")) MessageBox.Show("......", "Warning");
                pIDStr = pIDStr.Substring(0, pIDStr.Length - 1);
                pIDStr = pIDStr.Substring(0, pIDStr.LastIndexOf(",") + 1);
                pFTStr = pFTStr.Substring(0, pFTStr.Length - 1);
                pFTStr = pFTStr.Substring(0, pFTStr.LastIndexOf(",") + 1);
            }
        }


        /// <summary>
        /// According to the angle, always get the first one to turn left or right recursively.
        /// </summary>
        /// <param name="pOrigArcID">The starting arc ID, as the recursive bottom bound back condition</param>
        /// <param name="bOrigArcFT"></param>
        /// <param name="bOrigRight"></param>
        /// <param name="pCurrArcID"></param>
        /// <param name="bCurrArcFT"></param>
        /// <param name="pIDStr"></param>
        /// <param name="pFTStr"></param>
        /// <param name="bDirected"></param>
        /// <param name="pArcCls"></param>
        /// <param name="pCycle"></param>
        private void RecurCircum(int pOrigArcID, bool bOrigArcFT, bool bOrigRight, int pCurrArcID, bool bCurrArcFT, string pIDStr, string pFTStr,
            bool bDirected, IRouteArcClass pArcCls, ref IRouteCycle pCycle)
        {
            ArrayList pConnIDs, pConnFTs, pConnAngles;
            GetRankAngle(pArcCls.GetArc(pCurrArcID), bCurrArcFT, pArcCls, bDirected, out pConnIDs, out pConnFTs, out pConnAngles);
            if (pConnIDs.Count == 0) return;
            if (!bOrigRight) //from right to left(counterclockwisely), the angle changes from -180 to 180
            {
                pConnIDs.Reverse();
                pConnFTs.Reverse();
                pConnAngles.Reverse();
            }

            int pNextArcID = -1;
            bool bNextArcFT = false;
            IRouteArc pNextArc = null;
            ArrayList pConnIDs1, pConnFTs1, pConnAngles1;
            for (int i = 0; i < pConnIDs.Count; i++) //Always get the first one
            {
                pNextArcID = Convert.ToInt32(pConnIDs[i]);
                bNextArcFT = Convert.ToBoolean(pConnFTs[i]);

                if (pNextArcID == pOrigArcID)
                {
                    Arcs2Cycle(pIDStr, pFTStr, bOrigRight, ref pCycle, pArcCls); //MessageBox.Show("I am here, the largest cycle! :=)", "Information");
                    return;
                }

                pNextArc = pArcCls.GetArc(pNextArcID); IRoutePoint pt = pNextArc.GetPoint(pNextArc.PointCount - 1);
                GetRankAngle(pNextArc, !bNextArcFT, pArcCls, bDirected, out pConnIDs1, out pConnFTs1, out pConnAngles1);
                if (pConnIDs1.Count == 0)
                {
                    continue; //
                }

                //FlashDetection(pIDStr, pFTStr, pNextArcID, pArcCls);

                pIDStr += pNextArcID + ",";
                pFTStr += !bNextArcFT + ",";

                RecurCircum(pOrigArcID, bOrigArcFT, bOrigRight, pNextArcID, !bNextArcFT, pIDStr, pFTStr, bDirected, pArcCls, ref pCycle);
                if (pCycle != null) return;

                if (pIDStr.IndexOf(",") == pIDStr.LastIndexOf(",")) MessageBox.Show("......", "Warning");
                pIDStr = pIDStr.Substring(0, pIDStr.Length - 1);
                pIDStr = pIDStr.Substring(0, pIDStr.LastIndexOf(",") + 1);
                pFTStr = pFTStr.Substring(0, pFTStr.Length - 1);
                pFTStr = pFTStr.Substring(0, pFTStr.LastIndexOf(",") + 1);
            }
        }

        private bool HitPath(string pIDStr, IRouteArc pNextArc, bool bNextArcFT)
        {
            int pConnNum = 0;
            if (bNextArcFT)
                pConnNum = pNextArc.FromConnCount;
            else
                pConnNum = pNextArc.ToConnCount;

            string pTestStr = "," + pIDStr;

            int pConnArcID = -1;
            for (int i = 0; i < pConnNum; i++)
            {
                pConnArcID = pNextArc.GetConnID(i, bNextArcFT);
                if (pTestStr.IndexOf("," + pConnArcID + ",") != -1)
                    return true;
            }

            return false;
        }

        private void Arcs2Cycle(string pIDStr, string pFTStr, bool bOrigRight, ref IRouteCycle pCycle, IRouteArcClass pArcCls)
        {
            pCycle = new RouteCycle();
            ArrayList tmpLst1 = Str2Lst(pIDStr);
            ArrayList tmpLst2 = Str2Lst(pFTStr);

            for (int j = 0; j < tmpLst1.Count; j++)
            {
                int pArcID = Convert.ToInt32(tmpLst1[j]);
                IRouteArc pArc = pArcCls.GetArc(pArcID);
                pCycle.AddArc(pArcID, Convert.ToBoolean(tmpLst2[j]), pArc as IRouteBound, pArc.Length);
            }
            pCycle.Right = bOrigRight;
        }

        private ArrayList Str2Lst(string pStr)
        {
            ArrayList rtnLst = new ArrayList();
            string tmpStr = "";
            int j = 0, k = 0;
            while (pStr.IndexOf(",") > 0)
            {
                j = pStr.IndexOf(",") + 1;
                k = pStr.Length - j;
                tmpStr = pStr.Substring(0, pStr.IndexOf(","));
                rtnLst.Add(tmpStr);
                pStr = pStr.Substring(j, k);
            }
            return rtnLst;
        }

        #endregion

        #endregion
    }
}
